import { TokenizeMethod, Eat } from '../tokenizer'
import { RemarkParser } from '../RemarkParser'
import { Node, Point } from 'typed-unist'

const C_NEWLINE = '\n'
const C_TAB = '\t'
const C_SPACE = ' '
const C_HASH = '#'

const MAX_ATX_COUNT = 6

export const atxHeading: TokenizeMethod = function (this: RemarkParser, eat: Eat, value: string, silent?: boolean): Node | boolean {
  const self = this
  const settings = self.options
  let length = value.length + 1
  let index = -1
  const now: Point = eat.now()
  let subvalue = ''
  let content = ''
  let character
  let queue
  let depth

  /* Eat initial spacing. */
  while (++index < length) {
    character = value.charAt(index)

    if (character !== C_SPACE && character !== C_TAB) {
      index--
      break
    }

    subvalue += character
  }

  /* Eat hashes. */
  depth = 0

  while (++index <= length) {
    character = value.charAt(index)

    if (character !== C_HASH) {
      index--
      break
    }

    subvalue += character
    depth++
  }

  if (depth > MAX_ATX_COUNT) {
    return
  }

  if (
    !depth ||
    (!settings.pedantic && value.charAt(index + 1) === C_HASH)
  ) {
    return
  }

  length = value.length + 1

  /* Eat intermediate white-space. */
  queue = ''

  while (++index < length) {
    character = value.charAt(index)

    if (character !== C_SPACE && character !== C_TAB) {
      index--
      break
    }

    queue += character
  }

  /* Exit when not in pedantic mode without spacing. */
  if (
    !settings.pedantic &&
    queue.length === 0 &&
    character &&
    character !== C_NEWLINE
  ) {
    return
  }

  if (silent) {
    return true
  }

  /* Eat content. */
  subvalue += queue
  queue = ''
  content = ''

  while (++index < length) {
    character = value.charAt(index)

    if (!character || character === C_NEWLINE) {
      break
    }

    if (
      character !== C_SPACE &&
      character !== C_TAB &&
      character !== C_HASH
    ) {
      content += queue + character
      queue = ''
      continue
    }

    while (character === C_SPACE || character === C_TAB) {
      queue += character
      character = value.charAt(++index)
    }

    while (character === C_HASH) {
      queue += character
      character = value.charAt(++index)
    }

    while (character === C_SPACE || character === C_TAB) {
      queue += character
      character = value.charAt(++index)
    }

    index--
  }

  now.column += subvalue.length
  now.offset += subvalue.length
  subvalue += content + queue

  return eat(subvalue)({
    type: 'heading',
    depth,
    children: self.tokenizeInline(content, now),
  } as Node)
} as TokenizeMethod
