import { Doc } from 'typed-unified';
import { Node, Point, Position } from 'typed-unist';
import { VFile } from 'typed-vfile';
import { TokenizeMethod, Factory, Tokenize } from './tokenizer';
export interface RemarkParserOptions {
    position?: boolean;
    gfm?: boolean;
    commonmark?: boolean;
    footnotes?: boolean;
    pedantic?: boolean;
    blocks?: string[];
}
export interface InteruptRuleOptions {
    commonmark?: boolean;
    pedantic?: boolean;
}
export declare type InteruptRule = [string] | [string, InteruptRuleOptions];
export interface RemarkParserConstructor {
    new (doc: Doc, file: VFile): RemarkParser;
}
export interface RemarkParser {
    options: RemarkParserOptions;
    escape: string[];
    file: VFile;
    inList: boolean;
    inBlock: boolean;
    inLink: boolean;
    atStart: boolean;
    offset: {
        [key: number]: number;
    };
    toOffset: (potision: Point) => number;
    unescape: (value: string) => string;
    decode: (value: string, position: Point, handler: (value: string, location: Position, source: string) => void) => void;
    decodeRaw: (value: string, position: Point) => string;
    interruptParagraph: InteruptRule[];
    interruptList: InteruptRule[];
    interruptBlockquote: InteruptRule[];
    eof: Point;
    inlineMethods: string[];
    blockMethods: string[];
    blockTokenizers: {
        [key: string]: TokenizeMethod;
    };
    inlineTokenizers: {
        [key: string]: TokenizeMethod;
    };
    tokenizeBlock: Tokenize;
    tokenizeInline: Tokenize;
    tokenizeFactory: Factory;
    parse: (this: RemarkParser) => Node;
    exitStart: () => () => void;
    enterList: () => () => void;
    enterLink: () => () => void;
    enterBlock: () => () => void;
}
export declare const parserFactory: () => RemarkParserConstructor;
